package com.tzld.longarticle.recommend.server.service.exterior.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.response.CommonResponse;
import com.tzld.longarticle.recommend.server.mapper.crawler.GhDetailMapper;
import com.tzld.longarticle.recommend.server.model.param.GhDetailParam;
import com.tzld.longarticle.recommend.server.repository.model.GhDetail;
import com.tzld.longarticle.recommend.server.repository.model.GhDetailExample;
import com.tzld.longarticle.recommend.server.service.exterior.GhDetailService;
import com.tzld.longarticle.recommend.server.util.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import java.util.List;

@Slf4j
@Service
public class GhDetailServiceImpl implements GhDetailService {

    @Autowired
    private GhDetailMapper ghDetailMapper;

    @Override
    public CommonResponse<Page<GhDetail>> getGhDetailList(GhDetailParam param) {
        Page<GhDetail> page = new Page<>();
        page.setCurrentPage(param.getPageNum());
        page.setPageSize(param.getPageSize());
        GhDetailExample example = new GhDetailExample();
        example.setPage(page);
        long total = ghDetailMapper.countByExample(example);
        int totalSize = (int) (total % param.getPageSize() == 0 ? (total / param.getPageSize()) : (total / param.getPageSize() + 1));
        page.setTotalSize(totalSize);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        page.setObjs(ghDetails);
        return CommonResponse.success(page);
    }

    @Override
    public CommonResponse<Void> addGhDetail(GhDetail ghDetail) {
        ghDetailMapper.insert(ghDetail);
        return CommonResponse.success();
    }

    @Override
    public CommonResponse<Void> updateDetail(GhDetail ghDetail) {
        ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
        return CommonResponse.success();
    }

    @Override
    public CommonResponse<Void> refreshGhDetail() {
        GhDetailExample example = new GhDetailExample();
        example.createCriteria().andTypeEqualTo(1);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(ghDetails)) {
            return CommonResponse.success();
        }
        for (GhDetail ghDetail : ghDetails) {
            GhDetail updateGhDetail = getGhDetail(ghDetail);
            if (updateGhDetail == null) {
                continue;
            }
            log.info("getGhDetail = {}", ghDetail);
            ghDetailMapper.updateByPrimaryKeySelective(updateGhDetail);
        }
        return CommonResponse.success();
    }

    public GhDetail getGhDetail(GhDetail ghDetail) {
        String baseUrl = String.format("http://aigc-api.cybertogether.net/aigc/publish/account/algorithm/wechat/list?gh_id=%s", ghDetail.getId());
        WebClient webClient = WebClient.builder()
                .baseUrl(baseUrl)
                .build();
        Mono<String> mono = webClient.get().retrieve().bodyToMono(String.class);
        String block = mono.block();
        JSONArray jsonArray = JSONArray.parseArray(block);
        if (jsonArray != null && !jsonArray.isEmpty()) {
            JSONObject jsonObject = jsonArray.getJSONObject(0);
            log.info("jsonObject = {}", jsonObject);
            ghDetail.setGhName(jsonObject.getString("name"));
        }
        return null;
    }
}
